"""test_cli.py - Tests to confirm that the CLI works and that both running the module and
calling the entry point produce equivalent output.
"""

import subprocess
import sys
from pathlib import Path

BINDIR = Path(sys.executable).parent


def test_module_run():
    """Simple test for no error when running the module. Output is not validated."""
    subprocess.check_call([sys.executable, "-m", "trove_classifiers"])


def test_entry_point():
    """Simple test for no error when calling the entry point. Output is not validated."""
    subprocess.check_call(f"{BINDIR}/trove-classifiers")


def test_module_run_is_entry_point():
    """Compare that module run output is the same as entry point output."""
    module_run_proc = subprocess.run(
        [sys.executable, "-m", "trove_classifiers"],
        capture_output=True,
        encoding="utf-8",
    )
    entry_point_proc = subprocess.run(
        f"{BINDIR}/trove-classifiers", capture_output=True, encoding="utf-8"
    )
    assert module_run_proc.stdout == entry_point_proc.stdout



from io import open
from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="trove-classifiers",
    description="Canonical source for classifiers on PyPI (pypi.org).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/trove-classifiers",
    author="The PyPI Admins",
    author_email="admin@pypi.org",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Typing :: Typed",
    ],
    keywords="classifiers",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    package_data={"": ["py.typed"]},
    use_calver="%Y.%m.%d.%H",
    setup_requires=["calver"],
    entry_points={
        "console_scripts": ["trove-classifiers=trove_classifiers.__main__:cli"],
    },
)



class InvalidClassifier(Exception):
    pass


def trove_tester(classifiers, deprecated_classifiers):
    # Check the classifiers
    for classifier in classifiers:
        split = classifier.split(" :: ")

        # Check if this is an invalid top-level classifier
        if len(split) == 1:
            raise InvalidClassifier(f"Top-level classifier {classifier!r} is invalid")

        # Check if all parent classifiers are specified
        for i in range(2, len(split)):
            parent = " :: ".join(split[:i])
            if parent not in classifiers:
                raise InvalidClassifier(f"Classifier {parent!r} is missing")

        # Check the sub-classifiers
        for sub in split:
            # Check for whitespace
            if sub.strip().rstrip() != sub:
                raise InvalidClassifier(
                    "Classifiers starting or ending with whitespace are invalid"
                )

            # Check for private classifiers
            if sub.lower().startswith("private"):
                raise InvalidClassifier(
                    "Classifiers starting with 'Private' are invalid"
                )

            # Check for stray colons
            if ":" in sub:
                raise InvalidClassifier("Classifiers containing ':' are invalid")

    # Check the deprecated classifiers
    for deprecated_classifier, deprecated_by in deprecated_classifiers.items():
        # Check if the classifier is in both lists
        if deprecated_classifier in classifiers:
            raise InvalidClassifier(
                f"Classifier {deprecated_classifier!r} in both valid and deprecated classifiers"
            )

        # Check if all deprecated_by classifiers exist
        for new_classifier in deprecated_by:
            if new_classifier not in classifiers:
                raise InvalidClassifier(f"Classifier {new_classifier!r} does not exist")

    print("All classifiers passed :)")
